@@ -4,8 +4,8 @@ from fabric.api import *
 from fabric.colors import *
 from fabric.contrib.console import confirm
 from netaddr import IPNetwork, AddrFormatError
-from soil.fabric import eval_kwargs
-from soil.filesystem import temporary_directory, rsync
+from ground_soil.fabric import eval_kwargs
+from ground_soil.filesystem import temporary_directory, rsync
 from beanstalk import IDENTIFIER as beanstalk_identifier, BEANSTALK_LOCAL_BASE_PATH
 from beanstalk.tasks.utils import (set_verbose_level, load_role_settings as load_beanstalk_settings,
                                    load_web_servers, run_hooked_actions)
