@@ -3,8 +3,8 @@ import os
 from fabric.api import *
 from fabric.colors import *
 from fabric.contrib.console import confirm
-from soil.fabric import eval_kwargs, virtual_env
-from soil.filesystem import temporary_directory
+from ground_soil.fabric import eval_kwargs, virtual_env
+from ground_soil.filesystem import temporary_directory, edit_file
 from beanstalk import IDENTIFIER
 from beanstalk.decorators import beanstalk_role
 from beanstalk.tasks.utils import load_role_settings as load_beanstalk_settings, set_verbose_level
@@ -37,6 +37,14 @@ def project_venv_path(project_name):
     return os.path.join(project_base, 'venv/')
 
 
+def apps_pool_path():
+    beanstalk_settings = env.get('beanstalk_settings', None)
+    if beanstalk_settings is None:
+        abort('Load beanstalk settings first')
+
+    beanstalk_stack_base = beanstalk_settings['BEANSTALK_STACK_BASE']
+    return os.path.join(beanstalk_stack_base, 'apps')
+
 # Tasks
 #-----------------------------------------------------------------------------------------------------------------------
 
@@ -122,7 +130,7 @@ def setup(settings_path=None, **settings_patches):
     # Give beanstalk permission
     sudoer_file = '/etc/sudoers.d/beanstalk_reload_httpd'
     with open(sudoer_file, 'w') as f:
-        f.write('beanstalk ALL=(root) NOPASSWD:%s' % httpd_cmd_path)
+        f.write('beanstalk ALL=(root) NOPASSWD: %s' % httpd_cmd_path)
     local('chmod 440 %s' % sudoer_file)
 
     # Make beanstalk access ssh
@@ -133,6 +141,7 @@ def setup(settings_path=None, **settings_patches):
     print section_title('Deploy target is: %s' % beanstalk_stack_base)
     local('mkdir -p %s' % beanstalk_stack_base)
     local('mkdir -p %s' % os.path.join(beanstalk_stack_base, 'global-logs'))
+    local('echo "{}" >> %s' % os.path.join(beanstalk_stack_base, 'apps'))
     local('chmod o+rx %s' % beanstalk_stack_base)
     local('chown beanstalk:beanstalk %s' % beanstalk_stack_base)
 
@@ -176,6 +185,26 @@ def create_app(project_name, settings_path=None, **settings_patches):
             local('git add .gitignore')
             local('git commit -m "Init commit"')
 
+    # Add to pool
+    def editor(apps_pool):
+        # Get uwsgi port
+        uwsgi_port = 50000
+        # Add it
+        apps_pool[project_name] = {
+            'path': {
+                'base': project_base,
+                'source': project_source,
+                'logs': project_logs,
+                'venv': project_venv,
+            },
+            'uwsgi': {
+                'port': uwsgi_port,
+            },
+        }
+
+        return apps_pool
+    edit_file(apps_pool_path(), editor)
+
 
 @task
 @beanstalk_role('server')
@@ -196,6 +225,12 @@ def delete_app(project_name, settings_path=None, **settings_patches):
         print section_title('Remove app @ %s' % project_base)
         local('rm -rf %s' % project_base)
 
+        def editor(apps_pool):
+            if project_name in apps_pool:
+                del apps_pool[project_name]
+            return apps_pool
+        edit_file(apps_pool_path(), editor)
+
 
 @task
 @beanstalk_role('server')
@@ -238,8 +273,7 @@ def build_venv(project_name, settings_path=None, **settings_patches):
     """Build virtual env of target app.
     """
     settings_patches = eval_kwargs(settings_patches)
-    beanstalk_settings = load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
-    beanstalk_stack_base = beanstalk_settings['BEANSTALK_STACK_BASE']
+    load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
     set_verbose_level()
 
     # path
@@ -267,25 +301,15 @@ def app_info(project_name, settings_path=None, **settings_patches):
     beanstalk_settings = load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
     set_verbose_level()
 
+    with open(apps_pool_path(), 'r') as f:
+        apps_pool = json.loads(f.read())
+
     # Gather information
-    project_base = project_base_path(project_name)
-    project_source = project_source_path(project_name)
-    project_logs = project_logs_path(project_name)
-    project_venv = project_venv_path(project_name)
+    apps = apps_pool[project_name]
 
     if beanstalk_settings.get('USE_JSON', False):
-        result = {
-            'path': {
-                'base': project_base,
-                'source': project_source,
-                'logs': project_logs,
-                'venv': project_venv,
-            },
-            'uwsgi': {
-                #'port':  #TODO: uwsgi port
-            }
-        }
-        json_string = json.dumps(result)
+        result = apps
+        json_string = json.dumps(result, separators=(',', ':'))
 
         if beanstalk_settings.get('PRINT_JSON', True):
             print json_string
@@ -293,7 +317,17 @@ def app_info(project_name, settings_path=None, **settings_patches):
     else:
         print section_title('Beanstalk-stack app: %s' % project_name)
         print '-=' * 40
-        print 'Base   : %s' % project_base
-        print 'Source : %s' % project_base
-        print 'Logs   : %s' % project_base
-        print 'venv   : %s' % project_base
+        print 'Base   : %s' % apps['path']['base']
+        print 'Source : %s' % apps['path']['source']
+        print 'Logs   : %s' % apps['path']['logs']
+        print 'venv   : %s' % apps['path']['venv']
+
+
+@task
+@beanstalk_role('server')
+def list_apps(settings_path=None, **settings_patches):
+    """Get list of apps
+    """
+    settings_patches = eval_kwargs(settings_patches)
+    load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    set_verbose_level()
